@@ -129,6 +129,8 @@ struct _EBookPrivate {
 
 	gulong listener_signal;
 	gulong died_signal;
+
+	gint writable_idle_id;
 };
 
 
@@ -269,7 +271,9 @@ do_add_contact (gboolean          sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -296,7 +300,9 @@ do_add_contact (gboolean          sync,
 		e_contact_set (contact, E_CONTACT_UID, our_op->id);
 		g_free (our_op->id);
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -355,6 +361,8 @@ emit_async_add_contact_response (gpointer data)
 	if (op->cb.id)
 		op->cb.id (book, op->status, op->id, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	g_free (op->id);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
@@ -362,6 +370,7 @@ emit_async_add_contact_response (gpointer data)
 
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -377,9 +386,12 @@ e_book_response_add_contact (EBook       *book,
 
 	d(printf ("e_book_response_add_contact\n"));
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_add_contact: Cannot find operation ");
 		return;
 	}
@@ -402,6 +414,8 @@ e_book_response_add_contact (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -463,7 +477,9 @@ do_commit_contact (gboolean        sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -491,7 +507,9 @@ do_commit_contact (gboolean        sync,
 		g_free (our_op->id);
 
 		/* remove the op from the book's hash of operations */
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -571,6 +589,7 @@ do_get_supported_fields (gboolean             sync,
 		g_mutex_unlock (book->priv->mutex);
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_BUSY,
 			     _("book busy"));
+		return FALSE;
 	}
 
 	our_op = e_book_new_op (book, sync);
@@ -590,7 +609,9 @@ do_get_supported_fields (gboolean             sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -612,13 +633,15 @@ do_get_supported_fields (gboolean             sync,
 
 	if (sync) {
 		/* wait for something to happen (both cancellation and a
-		   successful response will notity us via our cv */
+		   successful response will notify us via our cv */
 		g_cond_wait (our_op->cond, our_op->mutex);
 
 		status = our_op->status;
 		*fields = our_op->list;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -672,6 +695,8 @@ emit_async_elist_response (gpointer data)
 	if (op->cb.elist)
 		op->cb.elist (book, op->status, op->elist, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	g_object_unref (op->elist);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
@@ -679,6 +704,7 @@ emit_async_elist_response (gpointer data)
 
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -692,9 +718,12 @@ e_book_response_get_supported_fields (EBook       *book,
 {
 	EBookOp *op;
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_supported_fields: Cannot find operation ");
 		return;
 	}
@@ -727,6 +756,8 @@ e_book_response_get_supported_fields (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -784,7 +815,9 @@ do_get_supported_auth_methods (gboolean             sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -809,7 +842,9 @@ do_get_supported_auth_methods (gboolean             sync,
 		status = our_op->status;
 		*auth_methods = our_op->list;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -862,9 +897,12 @@ e_book_response_get_supported_auth_methods (EBook                 *book,
 {
 	EBookOp *op;
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_supported_auth_methods: Cannot find operation ");
 		return;
 	}
@@ -895,6 +933,8 @@ e_book_response_get_supported_auth_methods (EBook                 *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -957,7 +997,9 @@ do_authenticate_user (gboolean        sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -981,7 +1023,9 @@ do_authenticate_user (gboolean        sync,
 
 		status = our_op->status;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -1095,7 +1139,9 @@ do_get_contact (gboolean sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -1121,7 +1167,9 @@ do_get_contact (gboolean sync,
 		status = our_op->status;
 		*contact = our_op->contact;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -1180,6 +1228,8 @@ emit_async_get_contact_response (gpointer data)
 	if (op->cb.contact)
 		op->cb.contact (book, op->status, op->contact, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	g_object_unref (op->contact);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
@@ -1187,6 +1237,7 @@ emit_async_get_contact_response (gpointer data)
 
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -1202,9 +1253,12 @@ e_book_response_get_contact (EBook       *book,
 
 	d(printf ("e_book_response_get_contact\n"));
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_contact: Cannot find operation ");
 		return;
 	}
@@ -1226,6 +1280,8 @@ e_book_response_get_contact (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -1602,6 +1658,8 @@ emit_async_get_book_view_response (gpointer data)
 	if (op->cb.book_view)
 		op->cb.book_view (book, op->status, op->view, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	g_object_unref (op->view);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
@@ -1609,6 +1667,7 @@ emit_async_get_book_view_response (gpointer data)
 
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -1625,9 +1684,12 @@ e_book_response_get_book_view (EBook       *book,
 
 	d(printf ("e_book_response_get_book_view\n"));
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_book_view: Cannot find operation ");
 		return;
 	}
@@ -1651,6 +1713,8 @@ e_book_response_get_book_view (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -1712,7 +1776,9 @@ do_get_contacts (gboolean sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -1739,7 +1805,9 @@ do_get_contacts (gboolean sync,
 		status = our_op->status;
 		*contacts = our_op->list;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -1797,13 +1865,18 @@ emit_async_get_contacts_response (gpointer data)
 
 	if (op->cb.list)
 		op->cb.list (book, op->status, op->list, op->closure);
+
+	g_mutex_lock (book->priv->mutex);
+
 	g_list_foreach (op->list, (GFunc)g_object_unref, NULL);
 	g_list_free (op->list);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
 						   GINT_TO_POINTER (op->idle_id));
+
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -1819,9 +1892,12 @@ e_book_response_get_contacts (EBook       *book,
 
 	EBookOp *op;
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_contacts: Cannot find operation ");
 		return;
 	}
@@ -1845,6 +1921,8 @@ e_book_response_get_contacts (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -1900,7 +1978,9 @@ do_get_changes (gboolean sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -1927,7 +2007,9 @@ do_get_changes (gboolean sync,
 		status = our_op->status;
 		*changes = our_op->list;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -1973,12 +2055,17 @@ emit_async_get_changes_response (gpointer data)
 
 	if (op->cb.list)
 		op->cb.list (book, op->status, op->list, op->closure);
+
+	g_mutex_lock (book->priv->mutex);
+
 	e_book_free_change_list (op->list);
 
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
 						   GINT_TO_POINTER (op->idle_id));
+
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -1993,9 +2080,12 @@ e_book_response_get_changes (EBook       *book,
 
 	EBookOp *op;
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_get_changes: Cannot find operation ");
 		return;
 	}
@@ -2016,6 +2106,8 @@ e_book_response_get_changes (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 void
@@ -2043,10 +2135,14 @@ emit_async_generic_response (gpointer data)
 	if (op->cb.status)
 		op->cb.status (book, op->status, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
 						   GINT_TO_POINTER (op->idle_id));
+
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -2059,9 +2155,12 @@ e_book_response_generic (EBook       *book,
 {
 	EBookOp *op;
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_generic: Cannot find operation ");
 		return;
 	}
@@ -2082,6 +2181,8 @@ e_book_response_generic (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 /**
@@ -2215,7 +2316,9 @@ do_open (gboolean sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -2241,7 +2344,9 @@ do_open (gboolean sync,
 
 		status = our_op->status;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		if (status == E_BOOK_ERROR_CANCELLED) {
 			/* Cancelled */
@@ -2306,18 +2411,26 @@ emit_async_open_response (gpointer data)
 
 	printf ("in async_open_response\n");
 
+	g_mutex_lock (book->priv->mutex);
+
 	if (op->status == E_BOOK_ERROR_OK)
 		book->priv->load_state = E_BOOK_SOURCE_LOADED;
 	else
 		book->priv->load_state = E_BOOK_SOURCE_NOT_LOADED;
 
+	g_mutex_unlock (book->priv->mutex);
+
 	if (op->cb.status)
 		op->cb.status (book, op->status, op->closure);
 
+	g_mutex_lock (book->priv->mutex);
+
 	book->priv->pending_idles = g_list_remove (book->priv->pending_idles,
 						   GINT_TO_POINTER (op->idle_id));
+
 	e_book_clear_op (book, op);
 
+	g_mutex_unlock (book->priv->mutex);
 	g_object_unref (book);
 
 	return FALSE;
@@ -2332,9 +2445,12 @@ e_book_response_open (EBook       *book,
 
 	printf ("in e_book_response_open\n");
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_open: Cannot find operation ");
 		return;
 	}
@@ -2355,6 +2471,8 @@ e_book_response_open (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
+
+	g_mutex_unlock (book->priv->mutex);
 }
 
 
@@ -2394,7 +2512,9 @@ do_remove (gboolean sync,
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		CORBA_exception_free (&ev);
 
@@ -2420,7 +2540,9 @@ do_remove (gboolean sync,
 
 		status = our_op->status;
 
+		g_mutex_lock (book->priv->mutex);
 		e_book_clear_op (book, our_op);
+		g_mutex_unlock (book->priv->mutex);
 
 		E_BOOK_CHECK_STATUS (status, error);
 	}
@@ -2473,9 +2595,12 @@ e_book_response_remove (EBook       *book,
 
 	d(printf ("e_book_response_remove\n"));
 
+	g_mutex_lock (book->priv->mutex);
+
 	op = e_book_get_op (book, opid);
 
 	if (op == NULL) {
+		g_mutex_unlock (book->priv->mutex);
 		g_warning ("e_book_response_remove: Cannot find operation ");
 		return;
 	}
@@ -2496,24 +2621,25 @@ e_book_response_remove (EBook       *book,
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
 	}
-}
 
-typedef struct
-{
-	EBook *book;
-	gboolean writable;
-}  EBookWritableData;
+	g_mutex_unlock (book->priv->mutex);
+}
 
 static gboolean
 e_book_idle_writable (gpointer data)
 {
-	EBookWritableData *write_data = data;
-	
-	g_signal_emit (G_OBJECT (write_data->book), e_book_signals [WRITABLE_STATUS], 0, write_data->writable);
+	EBook *book = data;
+	gboolean writable;
+
+	g_mutex_lock (book->priv->mutex);
+	writable = book->priv->writable;
+	book->priv->writable_idle_id = 0;
+	g_mutex_unlock (book->priv->mutex);
+
+	g_signal_emit (G_OBJECT (book), e_book_signals [WRITABLE_STATUS], 0, writable);
+
+	g_object_unref (book);
 
-	g_object_unref (write_data->book);
-	g_free (write_data);
-	
 	return FALSE;
 }
 
@@ -2523,7 +2649,6 @@ static void
 e_book_handle_response (EBookListener *listener, EBookListenerResponse *resp, EBook *book)
 {
 	EContact *contact;
-	EBookWritableData *write_data;
 
 	switch (resp->op) {
 	case CreateContactResponse:
@@ -2562,12 +2687,13 @@ e_book_handle_response (EBookListener *listener, EBookListenerResponse *resp, EB
 	case WritableStatusEvent: 
 		book->priv->writable = resp->writable;
 	
-		write_data = g_new0 (EBookWritableData, 1);
+		g_mutex_lock (book->priv->mutex);
+		if (book->priv->writable_idle_id == 0) {
+			g_object_ref (book);
+			book->priv->writable_idle_id = g_idle_add (e_book_idle_writable, book);
+		}
+		g_mutex_unlock (book->priv->mutex);
 
-		write_data->book = g_object_ref (book);
-		write_data->writable = book->priv->writable;
-		
-		g_idle_add (e_book_idle_writable, write_data);
 		break;
 	default:
 		g_error ("EBook: Unknown response code %d!\n",
@@ -2938,7 +3064,8 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 	*book = e_book_new_system_addressbook (&e);
 
 	if (!*book) {
-		g_propagate_error (error, e);
+		if (error)
+			g_propagate_error (error, e);
 		return FALSE;
 	}
 
@@ -2957,7 +3084,8 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 	if (!e_book_get_contact (*book, uid, contact, &e)) {
 		g_object_unref (*book);
 		*book = NULL;
-		g_propagate_error (error, e);
+		if (error)
+			g_propagate_error (error, e);
 		return FALSE;
 	}
 
@@ -3050,7 +3178,8 @@ e_book_set_default_source (ESource *source, GError **error)
 	uid = e_source_peek_uid (source);
 
 	if (!e_book_get_addressbooks (&sources, &err)) {
-		g_propagate_error (error, err);
+		if (error)
+			g_propagate_error (error, err);
 		return FALSE;
 	}
 
@@ -3078,7 +3207,8 @@ e_book_set_default_source (ESource *source, GError **error)
 	e_source_set_property (source, "default", "true");
 
 	if (!e_source_list_sync (sources, &err)) {
-		g_propagate_error (error, err);
+		if (error)
+			g_propagate_error (error, err);
 		return FALSE;
 	}
 
@@ -3228,7 +3358,8 @@ e_book_new_system_addressbook    (GError **error)
 	EBook *book;
 
 	if (!e_book_get_addressbooks (&sources, &err)) {
-		g_propagate_error (error, err);
+		if (error)
+			g_propagate_error (error, err);
 		return FALSE;
 	}
 
@@ -3267,8 +3398,10 @@ e_book_new_system_addressbook    (GError **error)
 		g_free (uri);
 	}
 
-	if (!book)
-		g_propagate_error (error, err);
+	if (!book) {
+		if (error)
+			g_propagate_error (error, err);
+	}
 
 	g_object_unref (sources);
 
@@ -3297,7 +3430,8 @@ e_book_new_default_addressbook   (GError **error)
 	EBook *book;
 
 	if (!e_book_get_addressbooks (&sources, &err)) {
-		g_propagate_error (error, err);
+		if (error)
+			g_propagate_error (error, err);
 		return FALSE;
 	}
 
@@ -3322,8 +3456,10 @@ e_book_new_default_addressbook   (GError **error)
 	else
 		book = e_book_new_system_addressbook (&err);
 
-	if (!book)
-		g_propagate_error (error, err);
+	if (!book) {
+		if (error)
+			g_propagate_error (error, err);
+	}
 
 	g_object_unref (sources);
 
@@ -3392,6 +3528,9 @@ e_book_dispose (GObject *object)
 
 		g_mutex_free (book->priv->mutex);
 
+		if (book->priv->writable_idle_id)
+			g_source_remove (book->priv->writable_idle_id);
+
 		g_free (book->priv);
 		book->priv = NULL;
 	}
